import json
import requests


class Client(object):

    def __init__(self, cnpj, days=None, token=None):
        self.cnpj = cnpj
        self.days = days
        self.token = token

    def get(self):
        url = 'https://www.receitaws.com.br/v1/cnpj/%s' % self.cnpj
        headers = {}

        if self.days and self.token:
            url = '%s/days/%s' % (url, self.days)
            headers.update({'Authorization': 'Bearer %s' % self.token})

        try:
            response = requests.get(url, headers=headers, timeout=70)
        except:
            return None
        if response.status_code != 200:
            return None
        return json.loads(response.content, encoding='utf-8')



import Queue
import threading

from receita.tools.client import Client


class Runner(object):
    """Client runner.

    We are keeping multiple requests running at the same time, but
    in a limited fashion. This will allow a better performance while
    getting data from the web service.

    This class assumes that the CNPJ list consists only of valid data
    and that the web service will always return valid json.
    """
    _CLIENT_LIMIT = 20

    def __init__(self, cnpjs, days=None, token=None):
        self._returned = 0
        self._stop = False
        self._list = cnpjs
        self._todo = Queue.Queue()
        self._results = Queue.Queue()
        self._days = days
        self._token = token

        for cnpj in self._list:
            self._todo.put(cnpj)

        self._threads = []
        for index in range(0, self._CLIENT_LIMIT):
            self._threads.append(threading.Thread(
                target=self.work,
                name='worker-%s' % index
            ))
            self._threads[-1].start()

    def __iter__(self):
        return self

    # Python 3 compatibility
    def __next__(self):
        return self.next()

    def next(self):
        if self._returned == len(self._list) or self._stop:
            self._wait_threads()
            raise StopIteration()

        self._returned = self._returned + 1
        return self._results.get(block=True)

    def stop(self):
        self._stop = True
        self._wait_threads()

    def _wait_threads(self):
        for thread in self._threads:
            thread.join()

    def work(self):
        while self._returned < len(self._list):
            if self._stop:
                break

            try:
                cnpj = self._todo.get(block=True, timeout=1)
            except Queue.Empty:
                continue

            data = Client(cnpj, self._days, self._token).get()
            if data:
                self._results.put((cnpj, data))
            else:
                self._todo.put(cnpj)



"""Module to execute all command line options to users.

This will be called to run various operations on companies data.
"""

import argparse
import sys

from receita.tools.get import Get
from receita.tools.build import Build


BASE_USAGE = """receita <command> [<args>]

the available commands are:
  get         download information about a list of companies
  build       create CSV files from the retrieved information"""


class Cli(object):

    def __init__(self):
        parser = argparse.ArgumentParser(
            usage=BASE_USAGE,
            epilog=(
                'You can get help for each command using the --help option '
                'for each of them.'
            )
        )
        parser.add_argument('command', help='command to run')
        args = parser.parse_args(sys.argv[1:2])
        if not hasattr(self, args.command):
            print 'unrecognized command\n'
            parser.print_help()
            exit(1)
        getattr(self, args.command)()

    def get(self):
        parser = argparse.ArgumentParser(
            prog='receita get',
            description='Download information about a list of companies.',
            epilog=(
                'The data is retrieved from ReceitaWS webservice on company '
                'information. Please, make sure you read its terms of usage '
                'and its documentation at https://www.receitaws.com.br on '
                'how the webservice works. You MAY be charged when using '
                'some features of this service.'
            )
        )
        parser.add_argument('list', help='CSV file with companies CNPJ')
        parser.add_argument(
            '-d',
            metavar="DAYS",
            dest='days',
            help='maximum data deprecation allowed in days')
        parser.add_argument(
            '--output',
            help='directory to save the output',
            default='data'
        )
        args = parser.parse_args(sys.argv[2:])

        # Execute
        Get(args.list, args.output, args.days).run()

    def build(self):
        parser = argparse.ArgumentParser(
            prog='receita build',
            description='Create CSV files from the retrieved information.')
        parser.add_argument(
            '--output',
            help='directory to save the generated CSV files',
            default='.'
        )
        parser.add_argument(
            '--input',
            help='directory to get input from',
            default='data'
        )
        args = parser.parse_args(sys.argv[2:])

        # Execute
        Build(args.input, args.output).run()


def main():
    Cli()


if __name__ == '__main__':
    main()



import glob
import json
import os
import sys
import unicodecsv


class BaseCSV(object):

    ERROR = 'ERROR'

    def __init__(self, output):
        path = os.path.join(output, self._filename + '.csv')
        self._f = open(path, 'w')

        self.writer = unicodecsv.DictWriter(
            self._f, fieldnames=self._fields,
            extrasaction='ignore')
        self.writer.writeheader()


class _CompaniesCSV(BaseCSV):

    _filename = 'companies'
    _fields = [
        'cnpj',
        'tipo',
        'abertura',
        'nome',
        'fantasia',
        'natureza_juridica',
        'logradouro',
        'numero',
        'complemento',
        'cep',
        'bairro',
        'municipio',
        'uf',
        'email',
        'telefone',
        'efr',
        'situacao',
        'data_situacao',
        'motivo_situacao',
        'situacao_especial',
        'data_situacao_especial',
    ]

    def visit(self, data):
        self.writer.writerow(data)


class _ActivitiesCSV(BaseCSV):

    _filename = 'activities'
    _fields = [
        'cnpj',
        'tipo',
        'codigo',
        'descricao',
    ]

    def visit(self, data):
        if data['status'] == self.ERROR:
            return

        for activity in data['atividade_principal']:
            self.writer.writerow({
                'cnpj': data['cnpj'],
                'tipo': 'principal',
                'codigo': activity['code'],
                'descricao': activity['text']
            })

        for activity in data['atividades_secundarias']:
            self.writer.writerow({
                'cnpj': data['cnpj'],
                'tipo': 'secundaria',
                'codigo': activity['code'],
                'descricao': activity['text']
            })


class _ActivitiesSeenCSV(BaseCSV):

    _filename = 'activities_seen'
    _fields = [
        'codigo',
        'descricao',
    ]

    def __init__(self, output):
        super(_ActivitiesSeenCSV, self).__init__(output)
        self._activities = {}

    def _process(self, activities):
        for activity in activities:
            if activity['code'] in self._activities:
                continue
            self._activities[activity['code']] = activity['text']
            self.writer.writerow({
                'codigo': activity['code'],
                'descricao': activity['text']
            })

    def visit(self, data):
        if data['status'] == self.ERROR:
            return
        self._process(data['atividade_principal'])
        self._process(data['atividades_secundarias'])


class Build(object):

    def __init__(self, input_, output):
        self.input = os.path.abspath(input_)
        self.output = os.path.abspath(output)

    def run(self):
        """Reads data from disk and generates CSV files."""
        # Try to create the directory
        if not os.path.exists(self.output):
            try:
                os.mkdir(self.output)
            except:
                print 'failed to create output directory %s' % self.output

        # Be sure it is a directory
        if not os.path.isdir(self.output):
            print 'invalid output directory %s' % self.output
            sys.exit(1)

        # Create the CSV handlers
        visitors = [
            _CompaniesCSV(self.output),
            _ActivitiesCSV(self.output),
            _ActivitiesSeenCSV(self.output)
        ]

        # Run by each company populating the CSV files
        for path in glob.glob(os.path.join(self.input, '*.json')):
            with open(path, 'r') as f:
                try:
                    data = json.load(f, encoding='utf-8')
                except ValueError:
                    continue

                for visitor in visitors:
                    visitor.visit(data)



import unicodecsv
import json
import re
import os
import sys

from progressbar import Bar, Counter, Percentage, ProgressBar, Timer

from receita.tools.runner import Runner


class Get(object):

    def __init__(self, file_, output, days):
        self.file = file_
        self.output = os.path.abspath(output)
        self.days = days
        self.token = os.environ.get('RWS_TOKEN')

        if self.days and not self.token:
            print('error: no token provided. Provide an access token using '
                  'the environment variable RWS_TOKEN.')
            sys.exit(1)

    def run(self):
        """Reads data from CNPJ list and write results to output directory."""
        self._assure_output_dir(self.output)
        companies = self.read()
        print '%s CNPJs found' % len(companies)

        pbar = ProgressBar(
            widgets=[Counter(), ' ', Percentage(), ' ', Bar(), ' ', Timer()],
            maxval=len(companies)).start()

        resolved = 0
        runner = Runner(companies, self.days, self.token)

        try:
            for data in runner:
                self.write(data)
                resolved = resolved + 1
                pbar.update(resolved)
        except KeyboardInterrupt:
            print '\naborted: waiting current requests to finish.'
            runner.stop()
            return

        pbar.finish()

    def read(self):
        """Reads data from the CSV file."""
        companies = []
        with open(self.file) as f:
            reader = unicodecsv.reader(f)
            for line in reader:
                if len(line) >= 1:
                    cnpj = self.format(line[0])
                    if self.valid(cnpj):
                        companies.append(cnpj)
        return companies

    def write(self, data):
        """Writes json data to the output directory."""
        cnpj, data = data

        path = os.path.join(self.output, '%s.json' % cnpj)
        with open(path, 'w') as f:
            json.dump(data, f, encoding='utf-8')

    def _assure_output_dir(self, dir):
        # Try to create the directory
        if not os.path.exists(dir):
            try:
                os.mkdir(dir)
            except:
                print 'failed to create output directory %s' % dir
                sys.exit(1)

        # Be sure it is a directory
        if not os.path.isdir(dir):
            print 'invalid output directory %s' % dir
            sys.exit(1)

    def format(self, cnpj):
        """Removes all symbols except digits."""
        return re.sub('\D', '', cnpj)

    def valid(self, cnpj):
        """Check if a CNPJ is valid.

        We should avoid sending invalid CNPJ to the web service as we know
        it is going to be a waste of bandwidth. Assumes CNPJ is a string.
        """
        if len(cnpj) != 14:
            return False

        tam = 12
        nums = cnpj[:tam]
        digs = cnpj[tam:]

        tot = 0
        pos = tam-7
        for i in range(tam, 0, -1):
            tot = tot + int(nums[tam-i])*pos
            pos = pos - 1
            if pos < 2:
                pos = 9
        res = 0 if tot % 11 < 2 else 11 - (tot % 11)
        if res != int(digs[0]):
            return False

        tam = tam + 1
        nums = cnpj[:tam]
        tot = 0
        pos = tam-7
        for i in range(tam, 0, -1):
            tot = tot + int(nums[tam-i])*pos
            pos = pos - 1
            if pos < 2:
                pos = 9
        res = 0 if tot % 11 < 2 else 11 - (tot % 11)
        if res != int(digs[1]):
            return False

        return True



from __future__ import print_function
from setuptools import setup
from setuptools.command.test import test as TestCommand
import os
import sys

import receita


class Tox(TestCommand):
    user_options = [('tox-args=', 'a', "Arguments to pass to tox")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.tox_args = None

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import tox
        import shlex
        args = self.tox_args
        if args:
            args = shlex.split(self.tox_args)
        errno = tox.cmdline(args=args)
        sys.exit(errno)


curdir = os.path.dirname(os.path.abspath(__file__))
with open(os.path.join(curdir, 'README.rst')) as readme:
    long_description = readme.read()

setup(
    # Basic info
    name='receita-tools',
    version=receita.__version__,
    url='http://github.com/vkruoso/receita-tools',
    license='MIT License',
    author='Vinicius K. Ruoso',
    author_email='vinicius.ruoso@gmail.com',
    description="Tools to manipulate Receita's company data.",
    long_description=long_description,

    # Details
    packages=['receita', 'receita.tools'],
    include_package_data=True,
    platforms='any',

    # On TLSv1.2 support:
    # https://github.com/kennethreitz/requests/blob/master/requests/packages/urllib3/contrib/pyopenssl.py
    # https://github.com/kennethreitz/requests/issues/3006
    # We are supporting TLSv1 for now, so it is not essential to have these
    # packages, but it is good to be prepared.
    install_requires=[
        'requests',
        'pyOpenSSL',  # needed for SSL support
        'ndg-httpsclient',  # needed for SSL support
        'pyasn1',   # needed for SSL support
        'unicodecsv',
        'progressbar'
    ],

    # Testing
    tests_require=['tox'],
    cmdclass={'test': Tox},

    # Scripts
    entry_points={
        'console_scripts': [
            'receita = receita.cli:main',
        ],
    },

    # Information
    classifiers=[
        'Programming Language :: Python',
        'Development Status :: 4 - Beta',
        'Natural Language :: English',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Utilities',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
